#include <string>
#include <iostream>
using namespace std;

class StackException {
   protected:
      string reason;
   public:
      StackException(const string &why):reason(why){}
      virtual string what(){return reason;}
};

class StackOverflowException : public StackException {
      public:
        StackOverflowException(const string &reason):StackException(reason){}
} ;

class StackUnderflowException : public StackException {
      public:
        StackUnderflowException(const string &reason):StackException(reason){}
} ;

template <class T,int size>
class Stack {
    protected:
       T hold[size];
       int top;
    public:
       Stack(){top=0;}
       void push(const T& v) throw(StackOverflowException);
       T& pop() throw (StackUnderflowException);   
} ;

template <class T,int size>
void Stack<T,size>::push(const T& v) throw(StackOverflowException) {
    if (top==size) throw StackOverflowException("Stack is full!");
    hold[top++]= v; 
}

template <class T,int size>
T& Stack<T,size>::pop() throw(StackUnderflowException) {
    if (top==0) throw StackUnderflowException("Stack is empty!");
    return hold[--top]; 
}

int main(){
	Stack<int,8> stack;
	int a[]={0,1,2,3,4,5,6,7,8,9};

	try {
	   for (int i=0;i<sizeof(a)/sizeof(int);i++)	
   	       stack.push(a[i]);
	}
	catch (StackOverflowException &e){
	   cout << e.what() ;	 
	}
	catch (StackUnderflowException &e){
	   cout << e.what() ;	 
	}
	catch (StackException &e){
	   cout << e.what() ;	 
	}

	try {
	   for (int i=0;i<sizeof(a)/sizeof(int);i++)	
               cout <<endl << stack.pop();	   
	}
	catch (StackOverflowException &e){
	   cout << e.what() ;	 
	}
	catch (StackUnderflowException &e){
	   cout << e.what() ;	 
	}
	catch (StackException &e){
	   cout << e.what() ;	 
	}

  return 0;
}
